#!/usr/bin/env python
# -*- coding: UTF8 -*-

#    pyLeech, downloads files listed in XML and HTML documents.
#    Copyright (C) 2008  Joakim Soderlund
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import urllib
import sys
import os

from sets import Set
from UserList import UserList
from sgmllib import SGMLParser

__author__ = "Joakim Soderlund"
__version__ = "1.0"
__date__ = "2008-05-02"
__copyright__ = "Copyright (C) 2008  Joakim Soderlund"
__license__ = "GPLv3"

class getHREFs(SGMLParser):
	"""Looks for links in HTML and SGML documents."""
	def reset(self):
		SGMLParser.reset(self)
		self.urls = []

	def start_a(self, attributes):
		for k, v in attributes:
			if k == "href":
				self.urls.append(v)

class getIMGs(SGMLParser):
	"""Looks for images in HTML and SGML documents."""
	def reset(self):
		SGMLParser.reset(self)
		self.urls = []

	def start_img(self, attributes):
		for k, v in attributes:
			if k == "src":
				self.urls.append(v)


class pyLeech(UserList):
	"""Handles a list with URLs."""
	def __init__(self):
		UserList.__init__(self)

	def getURLs(self, url, tag_href=False, tag_img=False):
		"""Looks for links in XML and HTML documents."""
		fileobject = urllib.urlopen(url)
		SGMLData = fileobject.read()
		fileobject.close()

		if tag_href:
			HREFLister = getHREFs()
			HREFLister.feed(SGMLData)
			HREFLister.close()
			self.data.extend(HREFLister.urls)
		if tag_img:
			IMGLister = getIMGs()
			IMGLister.feed(SGMLData)
			IMGLister.close()
			self.data.extend(IMGLister.urls)

		urls = self.data
		self.data = []
		for a in Set(urls):
			if re.search(r"^http\:\/\/", a):
				self.data.append(a)
			elif not re.search(r"^http\:\/\/", a) and re.search(r"\.\w*$", a):
				fullurl = "/".join(url.split("/")[:-1]) + "/" + a
				self.data.append(fullurl)

	def filterURLs(self, types):
		"""Takes a list of filetypes.
		Filters out all filetypes except for the specified filetypes."""
		urls = self.data
		self.data = []
		for a in types:
			a = a.lower()
			regex = re.compile("\.%s$" % a)

			for url in urls:
				if re.search(regex, url.lower()):
					self.data.append(url)

	def dlURLs(self, folder):
		"""Downloads the urls in self.data to the specified folder."""
		for a in self.data:
			filename = a.split("/")[-1]
			fullpath = "%s/%s" % (folder, filename)
			if not os.path.exists(fullpath):
				print "Downloading %s..." % filename
				urllib.urlretrieve(a, fullpath, self.progressBar)
			else:
				print "%s already exists." % fullpath

	def delPart(self, part):
		"""Removes a part of a URL."""
		counter = 0
		for a in self.data:
			self.data.remove(a)
			a = a.split("/")
			try:
				a.remove(part)
				counter += 1
			except ValueError:
				pass
			a = "/".join(a)
			self.data.append(a)
		if not counter == 0:
			self.delPart(part)

	def delDuplicates(self):
		"""Removes duplicate URLs from self.data."""
		dataset = Set(self.data)
		self.data = []
		for a in dataset:
			self.data.append(a)
		

	def progressBar(self, progress, bytesize, target):
		pass

if __name__ == "__main__":
	print """This is not yet an application, please run pyleech_gui.py."""
 
